@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2021 The Regents of the University of Michigan
+# Copyright (c) 2016-2022 The Regents of the University of Michigan
 # Part of GSD, released under the BSD 2-Clause License.
 
 """Read and write HOOMD schema GSD files.
@@ -248,6 +248,10 @@ class ParticleData(object):
             self.image = numpy.ascontiguousarray(self.image, dtype=numpy.int32)
             self.image = self.image.reshape([self.N, 3])
 
+        if (self.types is not None
+                and (not len(set(self.types)) == len(self.types))):
+            raise ValueError("Type names must be unique.")
+
 
 class BondData(object):
     """Store bond data chunks.
@@ -336,6 +340,10 @@ class BondData(object):
             self.group = numpy.ascontiguousarray(self.group, dtype=numpy.int32)
             self.group = self.group.reshape([self.N, self.M])
 
+        if (self.types is not None
+                and (not len(set(self.types)) == len(self.types))):
+            raise ValueError("Type names must be unique.")
+
 
 class ConstraintData(object):
     """Store constraint data.
@@ -807,7 +815,8 @@ class HOOMDTrajectory(object):
                              + '/' + name)
                 return False
 
-        if numpy.array_equiv(data, container._default_value[name]):
+        if numpy.array_equiv(data, container._default_value[name]) \
+                and not self.file.chunk_exists(frame=0, name=path + '/' + name):
             logger.debug('skipping data chunk, default value: ' + path + '/'
                          + name)
             return False
@@ -1048,15 +1057,16 @@ def open(name, mode='rb'):
     | ``'rb+'``        | Open an existing file for reading and       |
     |                  | writing.                                    |
     +------------------+---------------------------------------------+
-    | ``'wb'``         | Open a file for writing. Creates the file   |
-    |                  | if needed, or overwrites an existing file.  |
+    | ``'wb'``         | Open a file for reading and writing.        |
+    |                  | Creates the file if needed, or overwrites   |
+    |                  | an existing file.                           |
     +------------------+---------------------------------------------+
     | ``'wb+'``        | Open a file for reading and writing.        |
     |                  | Creates the file if needed, or overwrites   |
     |                  | an existing file.                           |
     +------------------+---------------------------------------------+
     | ``'xb'``         | Create a gsd file exclusively and opens it  |
-    |                  | for writing.                                |
+    |                  | for reading and writing.                    |
     |                  | Raise :py:exc:`FileExistsError`             |
     |                  | if it already exists.                       |
     +------------------+---------------------------------------------+
@@ -1065,11 +1075,10 @@ def open(name, mode='rb'):
     |                  | Raise :py:exc:`FileExistsError`             |
     |                  | if it already exists.                       |
     +------------------+---------------------------------------------+
-    | ``'ab'``         | Open an existing file for writing.          |
-    |                  | Does *not* create or overwrite existing     |
-    |                  | files.                                      |
+    | ``'ab'``         | Open an existing file for reading and       |
+    |                  | writing. Does *not* create or overwrite     |
+    |                  | existing files.                             |
     +------------------+---------------------------------------------+
-
     """
     if fl is None:
         raise RuntimeError("file layer module is not available")
